package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

import com.mojang.datafixers.kinds.Applicative;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import net.minecraftforge.registries.ForgeRegistries;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import com.google.common.base.Preconditions;
import java.util.Iterator;
import java.util.function.BiFunction;
import net.minecraft.network.chat.Style;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import net.minecraft.world.item.enchantment.Enchantment;

public class EnchantmentBonus extends GemBonus
{
    protected final Enchantment ench;
    protected final boolean mustExist;
    protected final boolean global;
    protected final Map<LootRarity, Integer> values;
    public static Codec<EnchantmentBonus> CODEC;
    
    public EnchantmentBonus(final GemClass gemClass, final Enchantment ench, final boolean mustExist, final boolean global, final Map<LootRarity, Integer> values) {
        super(Apotheosis.loc("enchantment"), gemClass);
        this.ench = ench;
        this.values = values;
        this.mustExist = mustExist;
        this.global = global;
    }
    
    @Override
    public Component getSocketBonusTooltip(final ItemStack gem, final LootRarity rarity) {
        final int level = this.values.get(rarity);
        String desc = "bonus." + this.getId() + ".desc";
        if (this.global) {
            desc += ".global";
        }
        else if (this.mustExist) {
            desc += ".mustExist";
        }
        final MutableComponent enchName = Component.m_237115_(this.ench.m_44704_());
        final Style style = this.ench.m_44700_(0).m_7383_();
        if (style.m_131135_() != null && style.m_131135_().m_131265_() != ChatFormatting.GRAY.m_126665_()) {
            enchName.m_130948_(style);
        }
        return (Component)Component.m_237110_(desc, new Object[] { level, Component.m_237115_("misc.apotheosis.level" + ((level > 1) ? ".many" : "")), enchName }).m_130940_(ChatFormatting.GREEN);
    }
    
    @Override
    public void getEnchantmentLevels(final ItemStack gemStack, final LootRarity rarity, final Map<Enchantment, Integer> enchantments) {
        final int level = this.values.get(rarity);
        if (this.global) {
            for (final Enchantment e : enchantments.keySet()) {
                enchantments.computeIfPresent(e, (key, val) -> (val > 0) ? (val + level) : 0);
            }
        }
        else if (this.mustExist) {
            enchantments.computeIfPresent(this.ench, (key, val) -> (val > 0) ? (val + level) : 0);
        }
        else {
            enchantments.merge(this.ench, level, Integer::sum);
        }
    }
    
    @Override
    public GemBonus validate() {
        Preconditions.checkNotNull((Object)this.ench, (Object)"Invalid DamageReductionBonus with null type");
        Preconditions.checkNotNull((Object)this.values, (Object)"Invalid DamageReductionBonus with null values");
        return this;
    }
    
    @Override
    public boolean supports(final LootRarity rarity) {
        return this.values.containsKey(rarity);
    }
    
    @Override
    public int getNumberOfUUIDs() {
        return 0;
    }
    
    public Codec<? extends GemBonus> getCodec() {
        return EnchantmentBonus.CODEC;
    }
    
    static {
        EnchantmentBonus.CODEC = (Codec<EnchantmentBonus>)RecordCodecBuilder.create(inst -> inst.group((App)GemBonus.gemClass(), (App)ForgeRegistries.ENCHANTMENTS.getCodec().fieldOf("enchantment").forGetter(a -> a.ench), (App)PlaceboCodecs.nullableField((Codec)Codec.BOOL, "must_exist", (Object)false).forGetter(a -> a.mustExist), (App)PlaceboCodecs.nullableField((Codec)Codec.BOOL, "global", (Object)false).forGetter(a -> a.global), (App)LootRarity.mapCodec(Codec.intRange(1, 127)).fieldOf("values").forGetter(a -> a.values)).apply((Applicative)inst, EnchantmentBonus::new));
    }
}
